/**
 * @author Kuitos
 * @since 2019-04-11
 */
import { SandBoxType } from '../../interfaces';
import * as css from './css';
import { patchLooseSandbox, patchStrictSandbox } from './dynamicAppend';
import patchHistoryListener from './historyListener';
import patchInterval from './interval';
import patchWindowListener from './windowListener';
export function patchAtMounting(appName, elementGetter, sandbox, scopedCSS, excludeAssetFilter, speedySandBox) {
    const basePatchers = [
        () => patchInterval(sandbox.proxy),
        () => patchWindowListener(sandbox.proxy),
        () => patchHistoryListener(),
    ];
    const patchersInSandbox = {
        [SandBoxType.LegacyProxy]: [
            ...basePatchers,
            () => patchLooseSandbox(appName, elementGetter, sandbox.proxy, true, scopedCSS, excludeAssetFilter),
        ],
        [SandBoxType.Proxy]: [
            ...basePatchers,
            () => patchStrictSandbox(appName, elementGetter, sandbox.proxy, true, scopedCSS, excludeAssetFilter, speedySandBox),
        ],
        [SandBoxType.Snapshot]: [
            ...basePatchers,
            () => patchLooseSandbox(appName, elementGetter, sandbox.proxy, true, scopedCSS, excludeAssetFilter),
        ],
    };
    return patchersInSandbox[sandbox.type]?.map((patch) => patch());
}
export function patchAtBootstrapping(appName, elementGetter, sandbox, scopedCSS, excludeAssetFilter, speedySandBox) {
    const patchersInSandbox = {
        [SandBoxType.LegacyProxy]: [
            () => patchLooseSandbox(appName, elementGetter, sandbox.proxy, false, scopedCSS, excludeAssetFilter),
        ],
        [SandBoxType.Proxy]: [
            () => patchStrictSandbox(appName, elementGetter, sandbox.proxy, false, scopedCSS, excludeAssetFilter, speedySandBox),
        ],
        [SandBoxType.Snapshot]: [
            () => patchLooseSandbox(appName, elementGetter, sandbox.proxy, false, scopedCSS, excludeAssetFilter),
        ],
    };
    return patchersInSandbox[sandbox.type]?.map((patch) => patch());
}
export { css };
//# sourceMappingURL=index.js.map